@@ -9,6 +9,7 @@ See http://kitchingroup.cheme.cmu.edu/pycse
 Copyright 2020, John Kitchin
 (see accompanying license files for details).
 """
+# pylint: disable=invalid-name
 
 import warnings
 import numpy as np
@@ -16,8 +17,11 @@ from scipy.stats.distributions import t
 from scipy.optimize import curve_fit
 from scipy.integrate import solve_ivp
 
+import numdifftools as nd
+
 # * Linear regression
 
+
 def polyfit(x, y, deg, alpha=0.05, *args, **kwargs):
     """Least squares polynomial fit with parameter confidence intervals.
 
@@ -40,14 +44,14 @@ def polyfit(x, y, deg, alpha=0.05, *args, **kwargs):
       bint is a 2D array of confidence intervals
       se is an array of standard error for each parameter.
     """
-
     # in vander, the second arg is the number of columns, so we have to add one
     # to the degree since there are N + 1 columns for a polynomial of order N
     X = np.vander(x, deg + 1)
     return regress(X, y, alpha, *args, **kwargs)
-    
-def regress(A, y, alpha=None, *args, **kwargs):
-    """Linear least squares regression with confidence intervals.
+
+
+def regress(A, y, alpha=0.05, *args, **kwargs):
+    r"""Linear least squares regression with confidence intervals.
 
     Solve the matrix equation \(A p = y\) for p.
 
@@ -87,12 +91,11 @@ def regress(A, y, alpha=None, *args, **kwargs):
       se is an array of standard error for each parameter.
 
     """
-
     # This is to silence an annoying FutureWarning.
-    if 'rcond' not in kwargs:
-        kwargs['rcond'] = None
-        
-    b, res, rank, s = np.linalg.lstsq(A, y, *args, **kwargs)
+    if "rcond" not in kwargs:
+        kwargs["rcond"] = None
+
+    b, _, _, _ = np.linalg.lstsq(A, y, *args, **kwargs)
 
     bint, se = None, None
 
@@ -110,25 +113,72 @@ def regress(A, y, alpha=None, *args, **kwargs):
         dC = np.diag(C)
 
         if (dC < 0.0).any():
-            warnings.warn('\n{0}\ndetected a negative number in your'
-                          'covariance matrix. Taking the absolute value'
-                          'of the diagonal. something is probably wrong'
-                          'with your data or model'.format(dC))
+            warnings.warn(
+                "\n{0}\ndetected a negative number in your"
+                "covariance matrix. Taking the absolute value"
+                "of the diagonal. something is probably wrong"
+                "with your data or model".format(dC)
+            )
             dC = np.abs(dC)
 
         se = np.sqrt(dC)  # standard error
 
-        sT = t.ppf(1.0 - alpha/2.0, n - k - 1)  # student T multiplier
+        sT = t.ppf(1.0 - alpha / 2.0, n - k - 1)  # student T multiplier
         CI = sT * se
 
         bint = np.array([(beta - ci, beta + ci) for beta, ci in zip(b, CI)])
 
     return (b, bint, se)
 
+
+def predict(X, y, pars, XX, alpha=0.05, ub=1e-5, ef=1.05):
+    """Prediction interval for linear regression.
+
+    Based on the delta method.
+
+    Parameters
+    ----------
+    X : known x-value array, one row for each y-point
+    y : known y-value array
+    pars : fitted parameters
+    XX : x-value array to make predictions for
+    alpha : confidence level, 95% = 0.05
+    ub : upper bound for smallest allowed Hessian eigenvalue
+    ef : eigenvalue factor for scaling Hessian
+
+    Returns
+    y, yint, pred_se
+    y : the predicted values
+    yint : an array of predicted confidence intervals
+    """
+
+    n = len(X)
+    npars = len(pars)
+    dof = n - npars
+
+    errs = y - X @ pars
+    sse = errs @ errs
+    rmse = sse / n
+
+    gprime = XX
+    hat = 2 * X.T @ X  # hessian
+    eps = max(ub, ef * np.linalg.eigvals(hat).min())
+
+    # Scaled Fisher information
+    I_fisher = rmse * np.linalg.pinv(hat + np.eye(npars) * eps)
+
+    pred_se = np.diag(gprime @ I_fisher @ gprime.T) ** 0.5
+    tval = t.ppf(1.0 - alpha / 2.0, dof)
+
+    yy = XX @ pars
+    return (yy, np.array([yy + tval * pred_se, yy - tval * pred_se]).T, pred_se)
+
+
 # * Nonlinear regression
 
+
 def nlinfit(model, x, y, p0, alpha=0.05, **kwargs):
-    """Nonlinear regression with confidence intervals.
+    r"""Nonlinear regression with confidence intervals.
 
     Parameters
     ----------
@@ -152,8 +202,10 @@ def nlinfit(model, x, y, p0, alpha=0.05, **kwargs):
     >>> X = np.array([0, 1, 2])
     >>> y = np.array([0, 2, 4])
     >>> nlinfit(f, X, y, [0, 1])
-    (array([  2.00000000e+00,  -2.18062024e-12]), array([[  2.00000000e+00,   2.00000000e+00],
-           [ -2.18315458e-12,  -2.17808591e-12]]), array([  1.21903752e-12,   1.99456367e-16]))
+    (array([  2.00000000e+00,  -2.18062024e-12]),
+     array([[  2.00000000e+00,   2.00000000e+00],
+           [ -2.18315458e-12,  -2.17808591e-12]]),
+     array([  1.21903752e-12,   1.99456367e-16]))
 
     Returns
     -------
@@ -164,17 +216,17 @@ def nlinfit(model, x, y, p0, alpha=0.05, **kwargs):
 
     """
     pars, pcov = curve_fit(model, x, y, p0=p0, **kwargs)
-    n = len(y)    # number of data points
+    n = len(y)  # number of data points
     p = len(pars)  # number of parameters
 
     dof = max(0, n - p)  # number of degrees of freedom
 
     # student-t value for the dof and confidence level
-    tval = t.ppf(1.0-alpha/2., dof)
+    tval = t.ppf(1.0 - alpha / 2.0, dof)
 
     SE = []
     pint = []
-    for i, p, var in zip(range(n), pars, np.diag(pcov)):
+    for p, var in zip(pars, np.diag(pcov)):
         sigma = var**0.5
         SE.append(sigma)
         pint.append([p - sigma * tval, p + sigma * tval])
@@ -182,8 +234,56 @@ def nlinfit(model, x, y, p0, alpha=0.05, **kwargs):
     return (pars, np.array(pint), np.array(SE))
 
 
+def nlpredict(X, y, model, loss, popt, xnew, alpha=0.05, ub=1e-5, ef=1.05):
+    """Prediction error for a nonlinear fit.
+
+    Parameters
+    ----------
+    model : model function with signature model(x, *p)
+    loss : loss function the model was fitted with loss(*p)
+    popt : the optimized paramters
+    xnew : x-values to predict at
+    alpha : confidence level, 95% = 0.05
+    ub : upper bound for smallest allowed Hessian eigenvalue
+    ef : eigenvalue factor for scaling Hessian
+
+    This function uses numdifftools for the Hessian and Jacobian.
+
+    Returns
+    -------
+
+    y, yint, se
+
+    y : predicted values
+    yint : prediction interval at alpha confidence interval
+    se : standard error of prediction
+    """
+
+    ypred = model(xnew, *popt)
+
+    hessp = nd.Hessian(lambda p: loss(*p))(popt)
+    # for making the Hessian better conditioned.
+    eps = max(ub, ef * np.linalg.eigvals(hessp).min())
+
+    sse = loss(*popt)
+    rmse = sse / len(y)
+    I_fisher = rmse * np.linalg.pinv(hessp + np.eye(len(popt)) * eps)
+
+    gprime = nd.Jacobian(lambda p: model(xnew, *p))(popt)
+
+    uncerts = np.sqrt(np.diag(gprime @ I_fisher @ gprime.T))
+    tval = t.ppf(1 - alpha / 2, len(y) - len(popt))
+
+    return [
+        ypred,
+        np.array([ypred + tval * uncerts, ypred - tval * uncerts]).T,
+        uncerts,
+    ]
+
+
 def Rsquared(y, Y):
-    """Return R^2, or coefficient of determination
+    """Return R^2, or coefficient of determination.
+
     y is a 1d array of observations.
     Y is a 1d array of predictions from a model.
 
@@ -193,20 +293,60 @@ def Rsquared(y, Y):
     """
     errs = y - Y
     SS_res = np.sum(errs**2)
-    SS_tot = np.sum((y - np.mean(y))**2)
+    SS_tot = np.sum((y - np.mean(y)) ** 2)
     return 1 - SS_res / SS_tot
-                    
 
 
-# * ivp
-
-
-def ivp(f, tspan, y0, *args, **kwargs):
-    """A friendlier version of solve_ivp.
+def bic(x, y, model, popt):
+    """Compute the Bayesian information criterion (BIC).
 
     Parameters
     ----------
+    model : function(x, *p) returns prediction for y
+    popt : optimal parameters
+    y : array, known y-values
 
+    Returns
+    -------
+    BIC : float
+
+    https://en.wikipedia.org/wiki/Bayesian_information_criterion#Gaussian_special_case
+    """
+
+    n = len(y)
+    k = len(popt)
+    rss = np.sum((model(x, *popt) - y) ** 2)
+    bic = n * np.log(rss / n) + k * np.log(n)
+    return bic
+
+
+def lbic(X, y, popt):
+    """Compute the Bayesian information criterion for a linear model.
+
+    Paramters
+    ---------
+    X : array of input variables in column form
+    y : known y values
+    popt : fitted parameters
+
+    Returns
+    -------
+    BIC : float
+    """
+
+    n = len(y)
+    k = len(popt)
+    rss = np.sum((X @ popt - y) ** 2)
+    bic = n * np.log(rss / n) + k * np.log(n)
+    return bic
+
+
+# * ivp
+def ivp(f, tspan, y0, *args, **kwargs):
+    """Solve an ODE initial value problem.
+
+    Parameters
+    ----------
     f : function
     callable y'(x, y) = f(x, y)
 
@@ -232,14 +372,14 @@ def ivp(f, tspan, y0, *args, **kwargs):
     t0, tf = tspan[0], tspan[-1]
 
     # make the max_step the smallest step in tspan, or what is in kwargs.
-    if 'max_step' not in kwargs:
-        kwargs['max_step'] = min(np.diff(tspan))
+    if "max_step" not in kwargs:
+        kwargs["max_step"] = min(np.diff(tspan))
 
-    if 'dense_output' not in kwargs:
-        kwargs['dense_output'] = True
+    if "dense_output" not in kwargs:
+        kwargs["dense_output"] = True
 
-    if 't_eval' not in kwargs:
-        kwargs['t_eval'] = tspan
+    if "t_eval" not in kwargs:
+        kwargs["t_eval"] = tspan
 
     sol = solve_ivp(f, (t0, tf), y0, *args, **kwargs)
 
@@ -252,6 +392,7 @@ def ivp(f, tspan, y0, *args, **kwargs):
 # * End
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import doctest
+
     doctest.testmod()
