@@ -1,36 +1,42 @@
+"""Module for use in Google Colab."""
+
 from datetime import datetime
 import glob
 import io
-from IPython import get_ipython
-from IPython.core.magic import register_line_magic
-from IPython.display import HTML, IFrame
-from nbconvert import HTMLExporter, PDFExporter
-import nbformat
 import os
-import requests
 import shlex
 import shutil
 import subprocess
 import tempfile
 from urllib.parse import urlparse
 
-from google.colab import drive
-from google.colab import files
-from googleapiclient.http import MediaIoBaseDownload
+from IPython import display
+from IPython.core.magic import register_line_magic
+from IPython.display import HTML, IFrame
+from nbconvert import HTMLExporter, PDFExporter
+import nbformat
 
-from google.colab import auth
-from googleapiclient.discovery import build
+import requests
 
+try:
+    from google.colab import drive
+    from google.colab import files
+    from googleapiclient.http import MediaIoBaseDownload
+
+    from google.colab import auth
+    from googleapiclient.discovery import build
+except ModuleNotFoundError:
+    pass
 
 DRIVE = None
 
 
 def gdrive():
-    '''Get the drive service, authenticate if needed.'''
+    """Get the drive service, authenticate if needed."""
     global DRIVE
     if DRIVE is None:
         auth.authenticate_user()
-        DRIVE = build('drive', 'v3')
+        DRIVE = build("drive", "v3")
     return DRIVE
 
 
@@ -38,26 +44,34 @@ def gdrive():
 # Utilities
 ##################################################################
 
-def aptupdate():
-    s = subprocess.run(['apt-get', 'update'],
-                       stdout=subprocess.PIPE,
-                       stderr=subprocess.PIPE)
-    if s.returncode != 0:
-        raise Exception(f'apt-get update failed.\n'
-                        f'{s.stdout.decode()}\n'
-                        f'{s.stderr.decode()}')
 
-    
-def aptinstall(apt_pkg):
-    '''Utility to install a package and check for success.'''
-    print(f'Installing {apt_pkg}. Please be patient.')
-    s = subprocess.run(['apt-get', 'install', apt_pkg],
-                       stdout=subprocess.PIPE,
-                       stderr=subprocess.PIPE)
+def aptupdate():
+    """Run apt-get update."""
+    s = subprocess.run(
+        ["apt-get", "update"], stdout=subprocess.PIPE, stderr=subprocess.PIPE
+    )
     if s.returncode != 0:
-        raise Exception(f'{apt_pkg} installation failed.\n'
-                        f'{s.stdout.decode()}\n'
-                        f'{s.stderr.decode()}')
+        raise Exception(
+            f"apt-get update failed.\n"
+            f"{s.stdout.decode()}\n"
+            f"{s.stderr.decode()}"
+        )
+
+
+def aptinstall(apt_pkg):
+    """Install a package and check for success."""
+    print(f"Installing {apt_pkg}. Please be patient.")
+    s = subprocess.run(
+        ["apt-get", "install", apt_pkg],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    if s.returncode != 0:
+        raise Exception(
+            f"{apt_pkg} installation failed.\n"
+            f"{s.stdout.decode()}\n"
+            f"{s.stderr.decode()}"
+        )
 
 
 # @register_line_magic
@@ -75,17 +89,18 @@ def aptinstall(apt_pkg):
 
 
 def current_notebook():
-    '''Returns current notebook name and file id.
+    """Return current notebook name and file id.
+
     from kora.drive.
-    '''
-    d = requests.get('http://172.28.0.2:9000/api/sessions').json()[0]
-    fid = d['path'].split('=')[1]
-    fname = d['name']
+    """
+    d = requests.get("http://172.28.0.2:9000/api/sessions").json()[0]
+    fid = d["path"].split("=")[1]
+    fname = d["name"]
     return fname, fid
 
 
 def notebook_string(fid):
-    '''Return noteook json data in string form for notebook at FID.'''
+    """Return noteook json data in string form for notebook at FID."""
     drive_service = gdrive()
     request = drive_service.files().get_media(fileId=fid)
     downloaded = io.BytesIO()
@@ -102,23 +117,27 @@ def notebook_string(fid):
     return ipynb
 
 
-def pdf_from_html(pdf=None, verbose=False, plotly=False, javascript_delay=10000):
-    '''Export the current notebook as a PDF.
+def pdf_from_html(
+    pdf=None, verbose=False, plotly=False, javascript_delay=10000
+):
+    """Export the current notebook as a PDF.
+
     pdf is the name of the PDF to export.
     plotly uses the plotly exporter
     The pdf is not saved in GDrive. Conversion is done from an HTML export.
     javascript_delay is in ms, and is how long to wait in wkhtmltopdf to let
     javascript, especially mathjax finish.
-    '''
+    """
     if verbose:
-        print('PDF via wkhtmltopdf')
+        print("PDF via wkhtmltopdf")
 
     fname, fid = current_notebook()
     ipynb = notebook_string(fid)
 
     if plotly:
-        subprocess.run(['pip', 'install', 'plotlyhtmlexporter'])
+        subprocess.run(["pip", "install", "plotlyhtmlexporter"])
         from plotlyhtmlexporter import PlotlyHTMLExporter
+
         exporter = PlotlyHTMLExporter()
     else:
         exporter = HTMLExporter()
@@ -127,7 +146,7 @@ def pdf_from_html(pdf=None, verbose=False, plotly=False, javascript_delay=10000)
     body, resources = exporter.from_notebook_node(nb)
 
     if verbose:
-        print(f'args: pdf={pdf}, verbose={verbose}')
+        print(f"args: pdf={pdf}, verbose={verbose}")
 
     if pdf is None:
         html = fname.replace(".ipynb", ".html")
@@ -136,7 +155,7 @@ def pdf_from_html(pdf=None, verbose=False, plotly=False, javascript_delay=10000)
         html = pdf.replace(".pdf", ".html")
 
     if verbose:
-        print(f'using html = {html}')
+        print(f"using html = {html}")
 
     tmpdirname = tempfile.TemporaryDirectory().name
 
@@ -145,54 +164,64 @@ def pdf_from_html(pdf=None, verbose=False, plotly=False, javascript_delay=10000)
 
     ahtml = os.path.join(tmpdirname, html)
     apdf = os.path.join(tmpdirname, pdf)
-    css = os.path.join(tmpdirname, 'custom.css')
+    css = os.path.join(tmpdirname, "custom.css")
 
-    with open(ahtml, 'w') as f:
+    with open(ahtml, "w") as f:
         f.write(body)
 
-    with open(css, 'w') as f:
-        f.write('\n'.join(resources['inlining']['css']))
+    with open(css, "w") as f:
+        f.write("\n".join(resources["inlining"]["css"]))
 
     aptupdate()
-    
-    if not shutil.which('xvfb-run'):        
-        aptinstall('xvfb')
 
-    if not shutil.which('wkhtmltopdf'):
-        aptinstall('wkhtmltopdf')
+    if not shutil.which("xvfb-run"):
+        aptinstall("xvfb")
+
+    if not shutil.which("wkhtmltopdf"):
+        aptinstall("wkhtmltopdf")
 
     if verbose:
-        print(f'Running with delay: {javascript_delay}')
+        print(f"Running with delay: {javascript_delay}")
 
-    s = subprocess.run(['xvfb-run', 'wkhtmltopdf',
-                        '--enable-javascript',
-                        '--no-stop-slow-scripts',
-                        '--javascript-delay', str(javascript_delay),
-                        ahtml, apdf],
-                       stdout=subprocess.PIPE,
-                       stderr=subprocess.PIPE)
+    s = subprocess.run(
+        [
+            "xvfb-run",
+            "wkhtmltopdf",
+            "--enable-javascript",
+            "--no-stop-slow-scripts",
+            "--javascript-delay",
+            str(javascript_delay),
+            ahtml,
+            apdf,
+        ],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
 
     if verbose and s.returncode != 0:
-        print(f'Conversion exited with non-zero status: {s.returncode}.\n'
-              f'{s.stdout.decode()}\n'
-              f'{s.stderr.decode()}')
+        print(
+            f"Conversion exited with non-zero status: {s.returncode}.\n"
+            f"{s.stdout.decode()}\n"
+            f"{s.stderr.decode()}"
+        )
 
     if os.path.exists(apdf):
         files.download(apdf)
     else:
-        print('no pdf found.')
+        print("no pdf found.")
         print(ahtml)
         print(apdf)
 
 
 def pdf_from_latex(pdf=None, verbose=False):
-    '''Export the notebook to PDF via LaTeX.
+    """Export the notebook to PDF via LaTeX.
+
     This is not fast because you have to install texlive.
     verbose is not used right now.
-    '''
-    print('PDF via LaTeX')
-    if not shutil.which('xelatex'):
-        aptinstall('texlive-xetex')
+    """
+    print("PDF via LaTeX")
+    if not shutil.which("xelatex"):
+        aptinstall("texlive-xetex")
 
     fname, fid = current_notebook()
     ipynb = notebook_string(fid)
@@ -215,56 +244,66 @@ def pdf_from_latex(pdf=None, verbose=False):
     if os.path.exists(apdf):
         os.unlink(apdf)
 
-    with open(apdf, 'wb') as f:
+    with open(apdf, "wb") as f:
         f.write(body)
 
     if os.path.exists(apdf):
         files.download(apdf)
     else:
-        print(f'{apdf} not found')
+        print(f"{apdf} not found")
 
 
-def pdf(line=''):
-    '''Line magic to export a colab to PDF.
-    You can have an optional arg -l to use LaTeX, defaults to html->PDF.
-    You can have an optional arg -p to use plotlyhtmlexporter
-    You can have an optional arg -d integer for a delay in seconds for the html to pdf.
-    
-    You can have an optional last argument for the filename of the pdf
-    '''
+def pdf(line=""):
+    """Line magic to export a colab to PDF.
+
+    You can have an optional arg -l to use LaTeX, defaults to html->PDF. This
+    takes longer to install, and may not work if you use non-standard LaTeX
+    code. I do not know how to add custom LaTeX packages to use arbitrary
+    commands.
+
+    You can have an optional arg -d integer for a delay in seconds for the html
+    to pdf. This is helpful when some equations are not rendering with
+    html->PDF. The rendering is done by MathJax, and notebooks with a lot of
+    equations take longer to render.
+
+    You can have an optional last argument for the filename of the pdf to save
+    to.
+
+    Known limitations:
+    1. If your notebook name doesn't end with .ipynb this does not work.
+
+    """
     args = shlex.split(line)
 
-    if args and args[-1].endswith('.pdf'):
+    if args and args[-1].endswith(".pdf"):
         pdf = args[-1]
     else:
         pdf = None
 
-    verbose = '-v' in args
+    verbose = "-v" in args
 
     if verbose:
-        print(f'%pdf args = {args}')
+        print(f"%pdf args = {args}")
 
-    if '-l' in args:
+    if "-l" in args:
         pdf_from_latex(pdf, verbose)
-        
-    
+
     else:
-        if '-d' in args:
-            i = args.index('-d')
+        if "-d" in args:
+            i = args.index("-d")
             # The delay should be in microseconds.
             delay = int(args[i + 1]) * 1000
         else:
             delay = 10000
-        plotly = '-p' in args
+        plotly = "-p" in args
         pdf_from_html(pdf, verbose, plotly, delay)
-                   
 
 
 # this is hackery so that CI works.
 # it is an error to do this when there is not IPython
 try:
     pdf = register_line_magic(pdf)
-except:
+except:  # noqa: E722
     pass
 
 
@@ -274,69 +313,74 @@ except:
 
 
 def fid_from_url(url):
-    '''Return a file ID for a file on GDrive from its url.'''
+    """Return a file ID for a file on GDrive from its url."""
     u = urlparse(url)
 
     # This is a typical sharing link
     # https://drive.google.com/file/d/1q_qE9RGdfV_8Vv3zuApf-LqXBwqo8HO2/view?usp=sharing
-    if (u.netloc == 'drive.google.com') and (u.path.startswith('/file/d/')):
-        return u.path.split('/')[3]
+    if (u.netloc == "drive.google.com") and (u.path.startswith("/file/d/")):
+        return u.path.split("/")[3]
 
     # This is a download link
     # https://drive.google.com/uc?id=1LLOGvaXsaEhUQXd7AmN_offy2IzNEu0K
-    elif (u.netloc == 'drive.google.com') and (u.path == '/uc'):
+    elif (u.netloc == "drive.google.com") and (u.path == "/uc"):
         q = u.query
         # I think this could have other things separated by &
-        qs = q.split('&')
+        qs = q.split("&")
         for item in qs:
-            if item.startswith('id='):
+            if item.startswith("id="):
                 return item[3:]
 
     # A colab url
     # https://colab.research.google.com/drive/1YcD5OXL-CNBO2h_OXZFb-mY6-LqgcLkB#scrollTo=0qkiF99z01pc
-    elif (u.netloc == 'colab.research.google.com'):
-        return u.path.split('/')[2]
+    elif u.netloc == "colab.research.google.com":
+        return u.path.split("/")[2]
 
     # 'https://docs.google.com/document/d/1lvDK2GisDM5aBnImtHNwOmLsU9jxg1NaPC46rB4bVqw/edit?usp=sharing'
-    elif (u.netloc == 'docs.google.com') and u.path.startswith('/document/d/'):
+    elif (u.netloc == "docs.google.com") and u.path.startswith("/document/d/"):
         p = u.path
-        p = p.replace('/document/d/', '')
-        p = p.replace('/edit', '')
+        p = p.replace("/document/d/", "")
+        p = p.replace("/edit", "")
         return p
 
     # https://docs.google.com/spreadsheets/d/1qSaBe73Pd8L3jJyOL68klp6yRArW7Nce/edit#gid=1923176268
-    elif (u.netloc == 'docs.google.com') and u.path.startswith('/spreadsheets/d/'):
+    elif (u.netloc == "docs.google.com") and u.path.startswith(
+        "/spreadsheets/d/"
+    ):
         p = u.path
-        p = p.replace('/spreadsheets/d/', '')
-        p = p.replace('/edit', '')
+        p = p.replace("/spreadsheets/d/", "")
+        p = p.replace("/edit", "")
         return p
 
-    #https://docs.google.com/presentation/d/1poP1gvWlfeZCR_5FsIzlRPMAYlBUR827wKPjbWGzW9M/edit#slide=id.p
-    elif (u.netloc == 'docs.google.com') and u.path.startswith('/presentation/d/'):
+    # https://docs.google.com/presentation/d/1poP1gvWlfeZCR_5FsIzlRPMAYlBUR827wKPjbWGzW9M/edit#slide=id.p
+    elif (u.netloc == "docs.google.com") and u.path.startswith(
+        "/presentation/d/"
+    ):
         p = u.path
-        p = p.replace('/presentation/d/', '')
-        p = p.replace('/edit', '')
+        p = p.replace("/presentation/d/", "")
+        p = p.replace("/edit", "")
         return p
 
     # https://drive.google.com/drive/u/0/folders/1aTs-_bhjT1GXy2P2hStzn31qAihRq2sl
-    elif (u.netloc == 'drive.google.com') and 'folders' in u.path:
-        return u.path.split('folders')[1][1:]
+    elif (u.netloc == "drive.google.com") and "folders" in u.path:
+        return u.path.split("folders")[1][1:]
 
     else:
-        raise Exception(f'Cannot parse {url} yet.')
+        raise Exception(f"Cannot parse {url} yet.")
 
 
-def gopen(fid_or_url_or_path, mode='r'):
-    '''Open a file on Gdrive by its ID, sharing link or path.
+def gopen(fid_or_url_or_path, mode="r"):
+    """Open a file on Gdrive by its ID, sharing link or path.
+
     Returns a file-like object you can read from.
     Note this reads the whole file into memory, so it may not
     be good for large files. Returns an io.StringIO if mode is "r"
     or io.BytesIO if mode is "rb".
-    '''
-    if mode not in ['r', 'rb']:
-        raise Exception(f'mode must be "r" or "rb"')
+    """
+    if mode not in ["r", "rb"]:
+        raise Exception('mode must be "r" or "rb"')
 
-    if fid_or_url_or_path.startswith('http'):
+    if fid_or_url_or_path.startswith("http"):
         fid = fid_from_url(fid_or_url_or_path)
     else:
         # it could be a path
@@ -345,7 +389,7 @@ def gopen(fid_or_url_or_path, mode='r'):
         else:
             # assume it is an fid
             fid = fid_or_url_or_path
-            print('fid: ', fid)
+            print("fid: ", fid)
 
     drive_service = gdrive()
     request = drive_service.files().get_media(fileId=fid)
@@ -359,11 +403,12 @@ def gopen(fid_or_url_or_path, mode='r'):
 
     # I prefer strings to bytes.
     downloaded.seek(0)
-    if mode == 'r':
+    if mode == "r":
         return io.TextIOWrapper(downloaded)
     else:
         return downloaded
 
+
 # Path utilities
 # This is tricky, paths are not deterministic in GDrive the way we are used to.
 # There is also some differences in My Drive and Shared drives, and files
@@ -372,41 +417,51 @@ def gopen(fid_or_url_or_path, mode='r'):
 
 def get_path(fid_or_url):
     """Return the path to an fid or url.
-    The path i's relative to the mount point."""
-    if fid_or_url.startswith('http'):
+
+    The path i's relative to the mount point.
+    """
+    if fid_or_url.startswith("http"):
         fid = fid_from_url(fid_or_url)
     else:
         fid = fid_or_url
 
     drive_service = gdrive()
-    x = drive_service.files().get(fileId=fid,
-                                  supportsAllDrives=True,
-                                  fields='parents,name').execute()
+    x = (
+        drive_service.files()
+        .get(fileId=fid, supportsAllDrives=True, fields="parents,name")
+        .execute()
+    )
 
-    dirs = [x['name']]  # start with the document name
+    dirs = [x["name"]]  # start with the document name
 
-    while x.get('parents', None):
+    while x.get("parents", None):
 
-        if len(x['parents']) > 1:
+        if len(x["parents"]) > 1:
             print(f'Warning, multiple parents found {x["parents"]}')
 
-        x = drive_service.files().get(fileId=x['parents'][0],
-                                      supportsAllDrives=True,
-                                      fields='id,parents,name').execute()
+        x = (
+            drive_service.files()
+            .get(
+                fileId=x["parents"][0],
+                supportsAllDrives=True,
+                fields="id,parents,name",
+            )
+            .execute()
+        )
 
-        if ('parents' not in x) and x['name'] == 'Drive':
+        if ("parents" not in x) and x["name"] == "Drive":
             # this means your file is in a shared drive I think.
-            drives = drive_service.drives().list().execute()['drives']
+            drives = drive_service.drives().list().execute()["drives"]
             for drv in drives:
-                if drv['id'] == x['id']:
-                    dirs += [drv['name'], 'Shared drives']
+                if drv["id"] == x["id"]:
+                    dirs += [drv["name"], "Shared drives"]
         else:
-            dirs += [x['name']]
+            dirs += [x["name"]]
 
-    if not os.path.isdir('/gdrive'):
-        drive.mount('/gdrive')
+    if not os.path.isdir("/gdrive"):
+        drive.mount("/gdrive")
 
-    dirs += ['/gdrive']
+    dirs += ["/gdrive"]
 
     dirs.reverse()
     p = os.path.sep.join(dirs)
@@ -415,7 +470,7 @@ def get_path(fid_or_url):
     # not always include the extension. We glob through matches to get the match
     # in this case.
     if not os.path.exists(p):
-        for f in glob.glob(f'{p}*'):
+        for f in glob.glob(f"{p}*"):
             if get_id(f) == fid:
                 return f
     else:
@@ -423,11 +478,11 @@ def get_path(fid_or_url):
 
 
 def get_id(path):
-    '''Given a path, return an id to it.'''
+    """Given a path, return an id to it."""
     drive_service = gdrive()
 
-    if not shutil.which('xattr'):
-        aptinstall('xattr')
+    if not shutil.which("xattr"):
+        aptinstall("xattr")
 
     path = os.path.abspath(path)
 
@@ -436,21 +491,21 @@ def get_id(path):
 
     elif os.path.isdir(path):
         # Strip the / gdrive off
-        path = path.split('/')[2:]
+        path = path.split("/")[2:]
 
-        if path[0] == 'My Drive' and len(path) == 1:
+        if path[0] == "My Drive" and len(path) == 1:
             return 0
 
-        if path[0] == 'My Drive':
-            drive_id = 'root'
-            id = 'root'
+        if path[0] == "My Drive":
+            drive_id = "root"
+            id = "root"
 
-        elif path[0] == 'Shared drives':
-            drives = drive_service.drives().list().execute()['drives']
+        elif path[0] == "Shared drives":
+            drives = drive_service.drives().list().execute()["drives"]
             for drv in drives:
-                if drv['name'] == path[1]:
-                    drive_id = drv['id']
-                    id = drv['id']
+                if drv["name"] == path[1]:
+                    drive_id = drv["id"]
+                    id = drv["id"]
                     break
 
         path = path[1:]
@@ -459,46 +514,49 @@ def get_id(path):
         for d in path:
             dsf = drive_service.files()
             args = dict(q=f"'{id}' in parents")
-            if drive_id != 'root':
-                args['corpora'] = 'drive'
-                args['supportsAllDrives'] = True
-                args['includeItemsFromAllDrives'] = True
-                args['driveId'] = drive_id
+            if drive_id != "root":
+                args["corpora"] = "drive"
+                args["supportsAllDrives"] = True
+                args["includeItemsFromAllDrives"] = True
+                args["driveId"] = drive_id
 
             file_list = dsf.list(**args).execute()
 
             found = False
-            for file1 in file_list.get('files', []):
-                if file1['name'] == d:
+            for file1 in file_list.get("files", []):
+                if file1["name"] == d:
                     found = True
-                    id = file1['id']
+                    id = file1["id"]
                     break
 
         if found:
             return id
 
         else:
-            raise Exception(f'Something went wrong with {path}')
+            raise Exception(f"Something went wrong with {path}")
 
     else:
-        raise Exception(f'{path} does not seem to be a file or directory')
+        raise Exception(f"{path} does not seem to be a file or directory")
 
 
 def get_link(path):
-    '''Returns a clickable link for path.'''
+    """Return a clickable link for path."""
     fid = get_id(os.path.abspath(path))
     drive_service = gdrive()
-    x = drive_service.files().get(fileId=fid,
-                                  supportsAllDrives=True,
-                                  fields='webViewLink').execute()
-    url = x.get('webViewLink', 'No web link found')
+    x = (
+        drive_service.files()
+        .get(fileId=fid, supportsAllDrives=True, fields="webViewLink")
+        .execute()
+    )
+    url = x.get("webViewLink", "No web link found")
     return HTML(f"<a href={url} target=_blank>{path}</a>")
 
 
 def gchdir(path=None):
-    '''Change working dir to path.
+    """Change working dir to path.
+
     if path is None, default to working directory of current notebook.
-    '''
+    """
     if path is None:
         path = os.path.dirname(get_path(current_notebook()[1]))
 
@@ -509,7 +567,8 @@ def gchdir(path=None):
 
 
 def gdownload(*FILES, **kwargs):
-    '''Download files. Each arg can be a path, or pattern.
+    """Download files. Each arg can be a path, or pattern.
+
     If you have more than one file, a zip is downloaded.
     You can specify a zip file name as a kwarg:
 
@@ -517,7 +576,7 @@ def gdownload(*FILES, **kwargs):
 
     The zip file will be deleted unless you use keep=True as a kwarg.
 
-    '''
+    """
     fd = []
     for f in FILES:
         for g in glob.glob(f):
@@ -526,8 +585,8 @@ def gdownload(*FILES, **kwargs):
     if (len(fd) == 1) and (os.path.isfile(fd[0])):
         files.download(fd[0])
     else:
-        if 'zip' in kwargs:
-            zipfile = kwargs['zip']
+        if "zip" in kwargs:
+            zipfile = kwargs["zip"]
         else:
             now = datetime.now()
             zipfile = now.strftime("%m-%d-%YT%H-%M-%S.zip")
@@ -535,14 +594,20 @@ def gdownload(*FILES, **kwargs):
         if os.path.exists(zipfile):
             os.unlink(zipfile)
 
-        s = subprocess.run(['zip', '-r', zipfile, *fd],
-                           stdout=subprocess.PIPE,
-                           stderr=subprocess.PIPE)
+        s = subprocess.run(
+            ["zip", "-r", zipfile, *fd],
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )
         if s.returncode != 0:
-            print(f'zip did not fully succeed:\n'
-                  f'{s.stdout.decode()}\n'
-                  f'{s.stderr.decode()}\n')
+            print(
+                f"zip did not fully succeed:\n"
+                f"{s.stdout.decode()}\n"
+                f"{s.stderr.decode()}\n"
+            )
         files.download(zipfile)
+
+
 #        if not kwargs.get('keep', False):
 #            os.unlink(zip)
 
@@ -550,23 +615,36 @@ def gdownload(*FILES, **kwargs):
 # Get to a shell
 ##################################################################
 def gconsole():
-    '''Open a shell in colab.
-    Adapted from https://github.com/airesearch-in-th/kora/blob/master/kora/console.py'''
+    """Open a shell in colab.
 
-    url = "https://github.com/gravitational/teleconsole/releases/download/0.4.0/teleconsole-v0.4.0-linux-amd64.tar.gz"
+    Adapted from
+    https://github.com/airesearch-in-th/kora/blob/master/kora/console.py
+    """
+    url = (
+        "https://github.com/gravitational/teleconsole/releases/download"
+        "/0.4.0/teleconsole-v0.4.0-linux-amd64.tar.gz"
+    )
     os.system(f"curl -L {url} | tar xz")  # download & extract
     os.system("mv teleconsole /usr/local/bin/")  # in PATH
 
     # Set PS1, directory
     with open("/root/.bashrc", "a") as f:
-        f.write('PS1="\e[1;36m\w\e[m# "\n')
+        f.write(r'PS1="\e[1;36m\w\e[m# "\n')
         f.write("cd /content \n")
-        f.write("PATH=/usr/local/nvidia/bin:/usr/local/cuda/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/tools/node/bin:/tools/google-cloud-sdk/bin:/opt/bin \n")
+        f.write(
+            "PATH=/usr/local/nvidia/bin:/usr/local/cuda/bin:/usr/local/sbin:"
+            "/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/tools/node/bin:"
+            "/tools/google-cloud-sdk/bin:/opt/bin \n"
+        )
 
-
-    process = subprocess.Popen("teleconsole", shell=True,
-                    stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    for i in range(6):
+    process = subprocess.Popen(
+        "teleconsole",
+        shell=True,
+        stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+    )
+    for _ in range(6):
         line = process.stdout.readline()
 
     url = line.decode().strip().split()[-1]
@@ -578,41 +656,32 @@ def gconsole():
 # Fancy outputs
 ##################################################################
 
+
 def gsuite(fid_or_url, width=1200, height=1000):
-    '''Return an iframe that renders the item in a colab.'''
+    """Return an iframe that renders the item in a colab."""
     drive_service = gdrive()
-    if fid_or_url.startswith('http'):
+    if fid_or_url.startswith("http"):
         url = fid_or_url
     else:
         # Assume we have an fid
-        x = drive_service.files().get(fileId=fid_or_url,
-                                      supportsAllDrives=True,
-                                      fields='webViewLink').execute()
-        url = x.get('webViewLink', 'No web link found.')
+        x = (
+            drive_service.files()
+            .get(
+                fileId=fid_or_url, supportsAllDrives=True, fields="webViewLink"
+            )
+            .execute()
+        )
+        url = x.get("webViewLink", "No web link found.")
 
-    display(HTML(f'''<a href="{url}" target="_blank">Link</a><br>'''))
+    display(HTML(f"""<a href="{url}" target="_blank">Link</a><br>"""))
 
     g = requests.get(url)
-    xframeoptions = g.headers.get('X-Frame-Options', '').lower()
-    if  xframeoptions in ['deny', 'sameorigin']:
-        print(f'X-Frame-Option = {xframeoptions}\nEmbedding in IFrame is not allowed for {url}.')
+    xframeoptions = g.headers.get("X-Frame-Options", "").lower()
+    if xframeoptions in ["deny", "sameorigin"]:
+        print(
+            f"X-Frame-Option = {xframeoptions}\n"
+            f"Embedding in IFrame is not allowed for {url}."
+        )
+        return None
     else:
         return IFrame(url, width, height)
-
-
-    # This is a good idea, but it would only work in colab?
-def read_csv(url, sheet_name='Sheet1', *args, **kwargs):
-    '''Return a dataframe for the Google Sheet at url.
-    sheet_name : the name of the sheet to read.
-    args and kwargs are passed to pd.read_csv
-
-    The url should be viewable by anyone with the link.
-    """
-    
-
-    adapted from https://towardsdatascience.com/read-data-from-google-sheets-into-pandas-without-the-google-sheets-api-5c468536550'''
-    fid = fid_from_url(url)
-    url = (f'https://docs.google.com/spreadsheets/d/{sheet_id}'
-           f'/gviz/tq?tqx=out:csv&sheet={sheet_name}')
-
-    return pd.read_csv(url, *args, **kwargs)
