@@ -1,6 +1,4 @@
-"""This module provides a dictionary of rgb values with keys of color names.
-
-"""
+"""This module provides a dictionary of rgb values with keys of color names."""
 
 color_table = """Aero	#7CB9E8	49%	73%	91%	206°	70%	70%	47%	91%
 Aero blue	#C9FFE5	79%	100%	90%	151°	100%	89%	21%	100%
@@ -621,7 +619,7 @@ Mustard	#FFDB58	100%	86%	35%	47°	100%	67%	65%	100%
 Myrtle green	#317873	19%	47%	45%	176°	42%	33%	59%	47%"""
 
 
-fields = [line.split('\t') for line in color_table.split('\n')]
+fields = [line.split("\t") for line in color_table.split("\n")]
 
 color_names = [field[0].lower() for field in fields]
 hex_codes = [field[1] for field in fields]
