@@ -1,37 +1,40 @@
+"""Provides classes to convert python data into org markup."""
 import IPython
 import tabulate
 
 
-class Heading(object):
+class Heading:
+    """An orgmode headline."""
+
     def __init__(self, title, level=1, tags=(), properties=None):
+        """Initialize a Heading."""
         self.title = title
         self.level = level
         self.tags = tags
         self.properties = properties
 
     def _repr_org(self):
-        s = '*' * self.level + ' ' + self.title
+        """Provide an org representation."""
+        s = "*" * self.level + " " + self.title
         if self.tags:
             s += f"  :{':'.join(self.tags)}:"
         if self.properties:
-            s += '\n:PROPERTIES:\n'
+            s += "\n:PROPERTIES:\n"
             for key in self.properties:
-                s += f':{key}: {self.properties[key]}\n'
-            s += ':END:'
-        return s + '\n'
+                s += f":{key}: {self.properties[key]}\n"
+            s += ":END:"
+        return s + "\n"
 
     def _repr_html(self):
-        '''HTML representation of a Heading'''
+        """HTML representation of a Heading."""
         return f"<h{self.level}>{self.title}</h{self.level}>"
 
     def _repr_mimebundle_(self, include, exclude, **kwargs):
-        """
-        repr_mimebundle should accept include, exclude and **kwargs
-        """
+        """Mimebundle representation of a Heading.
 
-        data = {'text/html': self._repr_html(),
-                'text/org': self._repr_org()
-                }
+        repr_mimebundle should accept include, exclude and **kwargs.
+        """
+        data = {"text/html": self._repr_html(), "text/org": self._repr_org()}
         if include:
             data = {k: v for (k, v) in data.items() if k in include}
         if exclude:
@@ -40,20 +43,23 @@ class Heading(object):
 
 
 class Keyword:
-    '''Keyword(key, value) -> #+key: value'''
+    """Keyword(key, value) -> #+key: value."""
+
     def __init__(self, key, value):
+        """Initialize a Keyword."""
         self.key = key
         self.value = value
 
     def _repr_org(self):
-        return f'#+{self.key}: {self.value}' + '\n'
+        """Provide org representation."""
+        return f"#+{self.key}: {self.value}" + "\n"
 
     def _repr_mimebundle_(self, include, exclude, **kwargs):
-        """
+        """Provide a mimebundle representation.
+
         repr_mimebundle should accept include, exclude and **kwargs
         """
-
-        data = {'text/org': self._repr_org()}
+        data = {"text/org": self._repr_org()}
         if include:
             data = {k: v for (k, v) in data.items() if k in include}
         if exclude:
@@ -62,19 +68,22 @@ class Keyword:
 
 
 class Comment:
-    '''Comment(text) -> # text'''
+    """Comment(text) -> # text."""
+
     def __init__(self, text):
+        """Initialize a comment."""
         self.text = text
 
     def _repr_org(self):
-        return f'# {self.text}' + '\n'
+        """Provide an org representation."""
+        return f"# {self.text}" + "\n"
 
     def _repr_mimebundle_(self, include, exclude, **kwargs):
-        """
+        """Provide a mimebundle representation.
+
         repr_mimebundle should accept include, exclude and **kwargs
         """
-
-        data = {'text/org': self._repr_org()}
+        data = {"text/org": self._repr_org()}
         if include:
             data = {k: v for (k, v) in data.items() if k in include}
         if exclude:
@@ -83,28 +92,37 @@ class Comment:
 
 
 class Org:
-    '''Org(text) -> text'''
+    """Org(text) -> text."""
+
     def __init__(self, text):
+        """Initialize an Org object."""
         self.text = text
 
     def _repr_org(self):
-        return self.text + '\n'
+        """Provide an org representation."""
+        return self.text + "\n"
 
     def _repr_mimebundle_(self, include, exclude, **kwargs):
-        """
+        """Provide a mimebundle representation.
+
         repr_mimebundle should accept include, exclude and **kwargs
         """
-
-        data = {'text/org': self._repr_org()}
+        data = {"text/org": self._repr_org()}
         if include:
             data = {k: v for (k, v) in data.items() if k in include}
         if exclude:
             data = {k: v for (k, v) in data.items() if k not in exclude}
-        return data    
+        return data
 
 
 class Figure:
+    """A Figure class for org.
+
+    It combines a filename, caption, name and attributes.
+    """
+
     def __init__(self, fname, caption=None, name=None, attributes=()):
+        """Initialize a figure."""
         self.fname = fname
         self.caption = caption
         self.name = name
@@ -113,24 +131,24 @@ class Figure:
     def _repr_org(self):
         s = []
         for backend, attrs in self.attributes:
-            s += [f'#+attr_{backend}: {attrs}']
+            s += [f"#+attr_{backend}: {attrs}"]
 
         if self.name:
-            s += [f'#+name: {self.name}']
+            s += [f"#+name: {self.name}"]
 
         if self.caption:
-            s += [f'#+caption: {self.caption}']
+            s += [f"#+caption: {self.caption}"]
 
-        s += [f'[[{self.fname}]]']
+        s += [f"[[{self.fname}]]"]
 
-        return '\n'.join(s) + '\n'
+        return "\n".join(s) + "\n"
 
     def _repr_mimebundle_(self, include, exclude, **kwargs):
-        """
-        repr_mimebundle should accept include, exclude and **kwargs
-        """
+        """Provide a mimebundle representation.
 
-        data = {'text/org': self._repr_org()}
+        repr_mimebundle should accept include, exclude and **kwargs.
+        """
+        data = {"text/org": self._repr_org()}
         if include:
             data = {k: v for (k, v) in data.items() if k in include}
         if exclude:
@@ -139,8 +157,12 @@ class Figure:
 
 
 class Table:
-    def __init__(self, data, headers=None, caption=None,
-                 name=None, attributes=()):
+    """A Table object for org."""
+
+    def __init__(
+        self, data, headers=None, caption=None, name=None, attributes=()
+    ):
+        """Initialize a table."""
         self.data = data
         self.headers = headers
         self.caption = caption
@@ -148,26 +170,27 @@ class Table:
         self.attributes = attributes
 
     def _repr_org(self):
+        """Provide an org representation."""
         s = []
         for backend, attributes in self.attributes:
-            s += [f'#+attr_{backend}: {attributes}']
+            s += [f"#+attr_{backend}: {attributes}"]
 
         if self.name:
-            s += [f'#+name: {self.name}']
+            s += [f"#+name: {self.name}"]
 
         if self.caption:
-            s += [f'#+caption: {self.caption}']
+            s += [f"#+caption: {self.caption}"]
 
-        s += [tabulate.tabulate(self.data, self.headers, tablefmt='orgtbl')]
+        s += [tabulate.tabulate(self.data, self.headers, tablefmt="orgtbl")]
 
-        return '\n'.join(s)
+        return "\n".join(s)
 
     def _repr_mimebundle_(self, include, exclude, **kwargs):
-        """
+        """Provide a mimebundle representation.
+
         repr_mimebundle should accept include, exclude and **kwargs
         """
-
-        data = {'text/org': self._repr_org()}
+        data = {"text/org": self._repr_org()}
         if include:
             data = {k: v for (k, v) in data.items() if k in include}
         if exclude:
@@ -177,13 +200,21 @@ class Table:
 
 # * Rich displays for org-mode
 
-class OrgFormatter(IPython.core.formatters.BaseFormatter):
-    format_type = IPython.core.formatters.Unicode('text/org')
-    print_method = IPython.core.formatters.ObjectName('_repr_org_')
 
-    
-ip = get_ipython()
-ip.display_formatter.formatters['text/org'] = OrgFormatter()
-ytv_f = ip.display_formatter.formatters['text/org']
-ytv_f.for_type_by_name('IPython.lib.display', 'YouTubeVideo',
-                       lambda V: f'{V.src}')    
+class OrgFormatter(IPython.core.formatters.BaseFormatter):
+    """A special formatter for org objects."""
+
+    format_type = IPython.core.formatters.Unicode("text/org")
+    print_method = IPython.core.formatters.ObjectName("_repr_org_")
+
+
+try:
+    ip = get_ipython()
+    ip.display_formatter.formatters["text/org"] = OrgFormatter()
+    ytv_f = ip.display_formatter.formatters["text/org"]
+    ytv_f.for_type_by_name(
+        "IPython.lib.display", "YouTubeVideo", lambda V: f"{V.src}"
+    )
+# get_ipython is not defined for tests I think.
+except NameError:
+    pass
