@@ -1,8 +1,10 @@
 #!/usr/bin/env python
-'''Module that runs as a script to redirect stderr to stdout.
-Output is designed for org-mode.'''
+"""Module that runs as a script to redirect stderr to stdout.
 
-from cStringIO import StringIO
+Output is designed for org-mode.
+"""
+
+from io import StringIO
 import sys
 
 content = sys.stdin.read()
@@ -24,19 +26,23 @@ sys.stdout = old_stdout
 sys.stderr = old_stderr
 
 
-s = '''{0}'''.format(out)
+s = """{0}""".format(out)
 
 if err:
-    s += '''
+    s += """
 #+STDERR:
 {0}
-'''.format(err)
+""".format(
+        err
+    )
 
 if exc:
-    s += '''
+    s += """
 #+EXCEPTIONS:
 {0}
-'''.format(exc)
+""".format(
+        exc
+    )
 
 # print final result to stdout
 print(s)
